@@ -2,6 +2,7 @@ use std::collections::HashMap;
 use std::fmt::{Display, Formatter};
 use std::str;
 
+use http::Uri;
 #[cfg(feature = "fetch")]
 use hyper::body::HttpBody;
 #[cfg(feature = "fetch")]
@@ -13,12 +14,11 @@ use hyper_tls::HttpsConnector;
 use lazy_static::lazy_static;
 use percent_encoding::percent_decode_str;
 use regex::Regex;
-use scraper::{Html, Selector};
-use url::Url;
 
 use crate::error::Error;
 
 lazy_static! {
+    static ref SCRIPT_REGEX: Regex = Regex::new(r#"(?s)<script type="text/javascript">(.+?)</script>"#).expect("cannot build script extracting regex");
     static ref VARIABLE_REGEX: Regex = Regex::new(r#"var\s*(\w+)\s*=\s*([\d+\-*/%]+);?"#).expect("cannot build variable matching regex");
     static ref LINK_GENERATOR_REGEX: Regex = Regex::new(r#"document\.getElementById\('dlbutton'\)\.href = "/d/(\w+)/"\s*\+\s*([\d\w\s+\-*/%()]+)\s*\+"/([/\w%.-]+)";?"#).expect("cannot build link generator regex");
 }
@@ -34,15 +34,12 @@ pub struct File {
 
 impl File {
     #[cfg(feature = "fetch")]
-    pub async fn fetch_and_parse(url: &str) -> Result<Self, Error> {
-        async fn fetch(url: &str) -> Result<Response<Body>, Error> {
+    pub async fn fetch_and_parse(uri: Uri) -> Result<Self, Error> {
+        async fn fetch(uri: Uri) -> Result<Response<Body>, Error> {
             let https = HttpsConnector::new();
             let client = Client::builder().build::<_, hyper::Body>(https);
             let response = client
-                .get(
-                    url.parse()
-                        .map_err(|err| Error::InvalidUrl { source: err })?,
-                )
+                .get(uri)
                 .await
                 .map_err(|err| Error::ContentFetchingFailure { source: err })?;
 
@@ -54,7 +51,7 @@ impl File {
             Ok(response)
         }
 
-        let mut response = fetch(url).await?;
+        let mut response = fetch(uri.clone()).await?;
         // Follow only one redirection.
         if response.status().is_redirection() {
             let location = response
@@ -62,7 +59,9 @@ impl File {
                 .get(header::LOCATION)
                 .ok_or(Error::RedirectionFailure)?
                 .to_str()
-                .map_err(|_| Error::RedirectionFailure)?;
+                .map_err(|_| Error::RedirectionFailure)?
+                .parse()
+                .map_err(|err| Error::InvalidUrl { source: err })?;
             response = fetch(location).await?;
         }
 
@@ -80,29 +79,27 @@ impl File {
         }
 
         Self::parse(
-            url,
+            &uri,
             str::from_utf8(&page_content)
-                .map_err(|err| Error::InvalidUtf8Content { source: err })?,
+                .map_err(|err| Error::InvalidUtf8PageContent { source: err })?,
         )
     }
 
-    pub fn parse(url: &str, page_content: &str) -> Result<Self, Error> {
+    pub fn parse(uri: &Uri, page_content: &str) -> Result<Self, Error> {
         let script_content = {
-            let document = Html::parse_document(page_content);
-            let selector =
-                Selector::parse("#lrbox script").map_err(|_| Error::InvalidCssSelector)?;
-
-            Ok(document
-                .select(&selector)
-                .find_map(|elem| {
-                    let html = elem.inner_html();
-                    LINK_GENERATOR_REGEX.is_match(&html).then(|| html)
-                })
-                .ok_or(Error::ScriptNotFound)?)
+            let mut content = Err(Error::ScriptNotFound);
+            for cap in SCRIPT_REGEX.captures_iter(page_content) {
+                let inner = cap.get(1).ok_or(Error::ScriptNotFound)?.as_str();
+                if inner.contains("document.getElementById('dlbutton')") {
+                    content = Ok(inner);
+                    break;
+                }
+            }
+            content
         }?;
 
         let vars = VARIABLE_REGEX
-            .captures_iter(&script_content)
+            .captures_iter(script_content)
             .map(|groups| {
                 Ok((
                     groups
@@ -124,7 +121,7 @@ impl File {
             .collect::<HashMap<_, _>>();
 
         let groups = LINK_GENERATOR_REGEX
-            .captures(&script_content)
+            .captures(script_content)
             .ok_or(Error::LinkGeneratorExtractionFailure)?;
         let expression = vars.iter().fold(
             groups
@@ -138,14 +135,27 @@ impl File {
             .map_err(|err| Error::LinkComputationFailure { source: err })? as i64;
 
         Ok(Self {
-            domain: Url::parse(url).unwrap().host_str().unwrap().to_owned(),
-            id: groups.get(1).unwrap().as_str().to_owned(),
+            domain: uri.host().ok_or(Error::DomainExtractionFailure)?.to_owned(),
+            id: groups
+                .get(1)
+                .ok_or(Error::FileIdExtractionFailure)?
+                .as_str()
+                .to_owned(),
             key,
-            name: percent_decode_str(groups.get(3).unwrap().as_str())
-                .decode_utf8()
-                .unwrap()
-                .to_string(),
-            encoded_name: groups.get(3).unwrap().as_str().to_owned(),
+            name: percent_decode_str(
+                groups
+                    .get(3)
+                    .ok_or(Error::FilenameExtractionFailure)?
+                    .as_str(),
+            )
+            .decode_utf8()
+            .map_err(|err| Error::InvalidUtf8Filename { source: err })?
+            .to_string(),
+            encoded_name: groups
+                .get(3)
+                .ok_or(Error::FilenameExtractionFailure)?
+                .as_str()
+                .to_owned(),
         })
     }
 
@@ -180,15 +190,29 @@ mod tests {
 
     #[tokio::test]
     async fn file_link() {
-        let file = super::File::fetch_and_parse("https://www3.zippyshare.com/v/CDCi2wVT/file.html").await.unwrap();
-        assert!(Regex::new(r#"https://(?:w+\d+\.)?zippyshare\.com/d/[\w\d]+/\d+/DOWNLOAD"#).unwrap().is_match(&file.link()));
+        let file = super::File::fetch_and_parse(
+            "https://www3.zippyshare.com/v/CDCi2wVT/file.html"
+                .parse()
+                .unwrap(),
+        )
+        .await
+        .unwrap();
+        assert!(
+            Regex::new(r#"https://(?:w+\d+\.)?zippyshare\.com/d/[\w\d]+/\d+/DOWNLOAD"#)
+                .unwrap()
+                .is_match(&file.link())
+        );
     }
-    
+
     #[tokio::test]
     async fn file_checksum() {
-        let file = super::File::fetch_and_parse("https://www3.zippyshare.com/v/CDCi2wVT/file.html")
-            .await
-            .unwrap();
+        let file = super::File::fetch_and_parse(
+            "https://www3.zippyshare.com/v/CDCi2wVT/file.html"
+                .parse()
+                .unwrap(),
+        )
+        .await
+        .unwrap();
 
         let https = HttpsConnector::new();
         let client = Client::builder().build::<_, hyper::Body>(https);
